# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseRunExecutionCountInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'batch_count': 'int',
        'finished_count': 'int',
        'interactive_count': 'int',
        'total_count': 'int'
    }

    attribute_map = {
        'batch_count': 'batch_count',
        'finished_count': 'finished_count',
        'interactive_count': 'interactive_count',
        'total_count': 'total_count'
    }

    def __init__(self, batch_count=None, finished_count=None, interactive_count=None, total_count=None, local_vars_configuration=None):  # noqa: E501
        """ResponseRunExecutionCountInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._batch_count = None
        self._finished_count = None
        self._interactive_count = None
        self._total_count = None
        self.discriminator = None

        if batch_count is not None:
            self.batch_count = batch_count
        if finished_count is not None:
            self.finished_count = finished_count
        if interactive_count is not None:
            self.interactive_count = interactive_count
        if total_count is not None:
            self.total_count = total_count

    @property
    def batch_count(self):
        """Gets the batch_count of this ResponseRunExecutionCountInfo.  # noqa: E501


        :return: The batch_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :rtype: int
        """
        return self._batch_count

    @batch_count.setter
    def batch_count(self, batch_count):
        """Sets the batch_count of this ResponseRunExecutionCountInfo.


        :param batch_count: The batch_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :type batch_count: int
        """

        self._batch_count = batch_count

    @property
    def finished_count(self):
        """Gets the finished_count of this ResponseRunExecutionCountInfo.  # noqa: E501


        :return: The finished_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :rtype: int
        """
        return self._finished_count

    @finished_count.setter
    def finished_count(self, finished_count):
        """Sets the finished_count of this ResponseRunExecutionCountInfo.


        :param finished_count: The finished_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :type finished_count: int
        """

        self._finished_count = finished_count

    @property
    def interactive_count(self):
        """Gets the interactive_count of this ResponseRunExecutionCountInfo.  # noqa: E501


        :return: The interactive_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :rtype: int
        """
        return self._interactive_count

    @interactive_count.setter
    def interactive_count(self, interactive_count):
        """Sets the interactive_count of this ResponseRunExecutionCountInfo.


        :param interactive_count: The interactive_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :type interactive_count: int
        """

        self._interactive_count = interactive_count

    @property
    def total_count(self):
        """Gets the total_count of this ResponseRunExecutionCountInfo.  # noqa: E501


        :return: The total_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :rtype: int
        """
        return self._total_count

    @total_count.setter
    def total_count(self, total_count):
        """Sets the total_count of this ResponseRunExecutionCountInfo.


        :param total_count: The total_count of this ResponseRunExecutionCountInfo.  # noqa: E501
        :type total_count: int
        """

        self._total_count = total_count

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseRunExecutionCountInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseRunExecutionCountInfo):
            return True

        return self.to_dict() != other.to_dict()



import atexit
import importlib
import io
import os
import threading
from typing import Optional

from deprecated import deprecated

import vessl
from vessl.util import logger
from vessl.util.audio import Audio
from vessl.util.common import get_module
from vessl.util.image import Image

MODE_TENSORFLOW2 = "mode-tf2"
MODE_OTHERS = "mode-others"

# Tensorboard event types
EVENT_TYPE_COMPRESSED_HISTOGRAMS = "distributions"
EVENT_TYPE_HISTOGRAMS = "histograms"
EVENT_TYPE_IMAGES = "images"
EVENT_TYPE_AUDIO = "audio"
EVENT_TYPE_SCALARS = "scalars"
EVENT_TYPE_TENSORS = "tensors"

# EventAccumulator argument (0 = keep all, N = keep N)
SIZE_GUIDANCE = {
    EVENT_TYPE_COMPRESSED_HISTOGRAMS: 1,
    EVENT_TYPE_IMAGES: 0,
    EVENT_TYPE_AUDIO: 0,
    EVENT_TYPE_SCALARS: 0,
    EVENT_TYPE_HISTOGRAMS: 1,
    EVENT_TYPE_TENSORS: 1,
}

READ_INTERVAL_IN_SEC = 1


class TensorboardCollector:
    """TensorboardCollector checks for new tensorboard events written to logdir and
    logs them to experiment metrics.

    There are two modes: `MODE_TENSORFLOW2` (TF2) and `MODE_OTHERS` (TF1, PyTorch, etc).
    `MODE_OTHERS` uses tensorboard's `EventAccumulator` to read from the event file.
    `MODE_TENSORFLOW2` uses tensorflow's `summary_iterator` because TF2 records events
      in a different way.
    """

    __slots__ = [
        "_exit",
        "_thread",
        "_tf_dict",
        "_event_accumulator",
    ]

    def start(self):
        raise Exception("Tensorflow is not supported in this version.")
        self._start_thread()

    @property
    def _mode(self):
        if hasattr(self, "_tf_dict"):
            return MODE_TENSORFLOW2
        if hasattr(self, "_event_accumulator"):
            return MODE_OTHERS
        return ""

    def _is_initialized(self):
        return self._mode in (MODE_TENSORFLOW2, MODE_OTHERS)

    def _start_thread(self):
        def exit_fn(*args):
            self._exit.set()
            self._thread.join()
            if not self._is_initialized():
                logger.warn("No tensorboard writer was detected during the run.")

        atexit.register(exit_fn)

        self._exit = threading.Event()
        self._thread = threading.Thread(target=self._thread_body, daemon=True)
        self._thread.start()

    def _thread_body(self):
        while not self._exit.is_set():
            self._main()
            self._exit.wait(timeout=READ_INTERVAL_IN_SEC)
        self._main()

    @deprecated(version="0.1.178", action="error")
    def _main(self):
        if not self._is_initialized():
            # Wait for initialization
            return

        if self._mode == MODE_TENSORFLOW2:
            self._main_tensorflow2()

        elif self._mode == MODE_OTHERS:
            self._main_others()

    def _main_tensorflow2(self):
        """Main thread body for MODE_TENSORFLOW2"""
        # Skip already seen events
        for _ in range(self._tf_dict["start_index"]):
            try:
                next(self._tf_dict["summary_iterator"])
            except StopIteration:
                continue

        events = []
        for e in self._tf_dict["summary_iterator"]:
            events.append(e)

        self._tf_dict["start_index"] += len(events)
        for e in events:
            self._handle_event(e)

    def _handle_event(self, event):
        for val_struct in event.summary.value:
            event_type = val_struct.metadata.plugin_data.plugin_name

            if event_type == EVENT_TYPE_SCALARS:
                event_value = self._tf_dict["make_ndarray_fn"](val_struct.tensor).item()

            elif event_type == EVENT_TYPE_IMAGES:
                PILImage = get_module(
                    "PIL.Image",
                    required='Pillow package is required. Run "pip install Pillow".',
                )
                # First two tensors are height and width, image bytestrings start
                # from third tensor.
                event_value = [
                    Image(data=PILImage.open(io.BytesIO(bs)), caption=val_struct.tag)
                    for bs in val_struct.tensor.string_val[2:]
                ]

            elif event_type == EVENT_TYPE_AUDIO:
                wave = get_module(
                    "wave", required='wave package is required. Run "pip install wave".'
                )
                np = get_module(
                    "numpy",
                    required='numpy package is required. Run "pip install numpy".',
                )

                event_value = []
                for bs in val_struct.tensor.string_val:
                    if not bs:
                        continue
                    wave_object = wave.open(io.BytesIO(bs))
                    data = np.frombuffer(
                        wave_object.readframes(wave_object.getnframes()),
                        dtype="float32",
                    )
                    vessl_audio = Audio(
                        data,
                        sample_rate=wave_object.getframerate(),
                        caption=val_struct.tag,
                    )
                    event_value.append(vessl_audio)

            else:
                continue

            payload = {val_struct.tag: event_value}
            vessl.log(payload=payload, step=event.step, ts=event.wall_time)

    def _main_others(self):
        """Main thread body for MODE_OTHERS"""
        try:
            self._event_accumulator.Reload()  # Loads each event at most once
        except Exception:
            # No metrics have been saved yet
            return

        event_tags = self._event_accumulator.Tags()
        # ex. {"scalars": ["loss", "accuracy"], "images": ["caption"]}

        self._handle_scalars(event_tags[EVENT_TYPE_SCALARS])
        self._handle_images(event_tags[EVENT_TYPE_IMAGES])
        self._handle_audios(event_tags[EVENT_TYPE_AUDIO])

    def _handle_scalars(self, tags):
        for tag in tags:
            scalars = self._event_accumulator.Scalars(tag)
            self._flush_scalars(tag)  # Flush right away
            for scalar in scalars:
                self._log_scalar(tag, scalar)

    def _flush_scalars(self, tag):
        with self._event_accumulator.scalars._mutex:
            scalars = self._event_accumulator.scalars
        with scalars._buckets[tag]._mutex:
            scalars._buckets[tag].items = []

    def _log_scalar(self, tag, scalar):
        payload = {tag: scalar.value}
        vessl.log(payload=payload, step=scalar.step, ts=scalar.wall_time)

    def _handle_images(self, tags):
        for tag in tags:
            images = self._event_accumulator.Images(tag)
            self._flush_images(tag)  # Flush right away
            for image in images:
                self._log_image(tag, image)

    def _flush_images(self, tag):
        with self._event_accumulator.images._mutex:
            images = self._event_accumulator.images
        with images._buckets[tag]._mutex:
            images._buckets[tag].items = []

    def _log_image(self, tag, image):
        PILImage = get_module(
            "PIL.Image",
            required='Pillow package is required. Run "pip install Pillow".',
        )

        vessl_image = Image(data=PILImage.open(io.BytesIO(image.encoded_image_string)), caption=tag)

        payload = {tag: vessl_image}
        vessl.log(payload=payload, step=image.step, ts=image.wall_time)

    def _handle_audios(self, tags):
        for tag in tags:
            audios = self._event_accumulator.Audio(tag)
            self._flush_audios(tag)  # Flush right away
            for audio in audios:
                self._log_audio(tag, audio)

    def _flush_audios(self, tag):
        with self._event_accumulator.audios._mutex:
            audios = self._event_accumulator.audios
        with audios._buckets[tag]._mutex:
            audios._buckets[tag].items = []

    def _log_audio(self, tag, audio):
        wave = get_module("wave", required='wave package is required. Run "pip install wave".')
        np = get_module("numpy", required='numpy package is required. Run "pip install numpy".')

        wave_object = wave.open(io.BytesIO(audio.encoded_audio_string))
        data = np.frombuffer(wave_object.readframes(wave_object.getnframes()), dtype="float32")
        vessl_audio = Audio(data, sample_rate=int(audio.sample_rate), caption=tag)

        payload = {tag: vessl_audio}
        vessl.log(payload=payload, step=audio.step, ts=audio.wall_time)

    def set_logdir(self, logdir, mode):
        """Called when tensorboard writer is detected. Only the first logdir will be used."""
        if not self._is_initialized():
            logger.info(f"Tensorboard logdir detected: {logdir}.")
            if mode == MODE_TENSORFLOW2:
                self._initialize_mode_tensorflow2(logdir)
            elif mode == MODE_OTHERS:
                self._initialize_mode_others(logdir)
        else:
            logger.debug(f"Cannot use multiple tensorboard logdirs. {logdir} will be ignored.")

    @deprecated(version="0.1.178", action="error")
    def _initialize_mode_tensorflow2(self, logdir):
        logger.error("Tensorflow is not supported in this version.")
        try:
            from tensorflow import make_ndarray
            from tensorflow.compat.v1.train import summary_iterator
        except Exception:
            logger.error(f"Could not import tensorflow. Failed to integrate tensorboard.")
            return

        # Get most recent event file. The event file for this run will have been just
        # created (in `tf.python.ops.gen_summary_ops.create_summary_file_writer`).
        path = mtime = None
        for dirpath, _, file_names in os.walk(logdir):
            for file_name in file_names:
                file_path = os.path.join(dirpath, file_name)
                file_mtime = os.path.getmtime(file_path)
                if "tfevents" in file_name and (mtime is None or mtime < file_mtime):
                    path, mtime = file_path, file_mtime

        if path is None:
            logger.info(f"No event file found in logdir: {logdir}.")
            return

        # Dict for MODE_TENSORFLOW2 objects
        self._tf_dict = {
            "make_ndarray_fn": make_ndarray,
            "summary_iterator": summary_iterator(path),
            "start_index": 0,
        }

    def _initialize_mode_others(self, logdir):
        try:
            from tensorboard.backend.event_processing.event_accumulator import (
                EventAccumulator,
            )
        except Exception:
            logger.error(
                "Could not import tensorboard. (Please install using `pip install tensorboard` first.) Failed to integrate tensorboard.",
            )
        self._event_accumulator = EventAccumulator(logdir, size_guidance=SIZE_GUIDANCE)

        try:
            self._event_accumulator.Reload()
        except Exception:
            # Logdir doesn't exist yet
            return

        # Disregard preexisting tensorboard logs
        for type, tags in self._event_accumulator.Tags().items():
            if type == EVENT_TYPE_SCALARS:
                for tag in tags:
                    self._flush_scalars(tag)
            if type == EVENT_TYPE_IMAGES:
                for tag in tags:
                    self._flush_images(tag)
            if type == EVENT_TYPE_AUDIO:
                for tag in tags:
                    self._flush_audios(tag)

    def close(self):
        """Joins thread to stop collecting. Used by tests."""
        self._exit.set()
        self._thread.join()
        return


# Global variable to ensure only one `TensorboardCollector` is active
tensorboard_collector_instance: Optional[TensorboardCollector] = None


def integrate_tensorboard():
    """Integrate tensorboard

    This method is called from `vessl.init` and can also be called manually."""
    global tensorboard_collector_instance

    if tensorboard_collector_instance is not None:
        logger.warning("Tensorboard is already enabled.")
        return

    tensorboard_collector_instance = TensorboardCollector()
    tensorboard_collector_instance.start()

    _patch_create_summary_file_writer(tensorboard_collector_instance)
    _patch_event_file_writers(tensorboard_collector_instance)


def _patch_create_summary_file_writer(tc):
    module_name = "tensorflow.python.ops.gen_summary_ops"
    try:
        module = importlib.import_module(module_name)
    except Exception as e:
        logger.debug(f"Module {module_name} not found. Skipping patch...")
        return

    _create_summary_file_writer = module.create_summary_file_writer

    def custom_create_summary_file_writer(*args, **kwargs):
        ret = _create_summary_file_writer(*args, **kwargs)
        logdir = (
            kwargs["logdir"].numpy().decode("utf8")
            if hasattr(kwargs["logdir"], "numpy")
            else kwargs["logdir"]
        )
        # Calling `set_logdir` after `_create_summary_file_writer` ensures that the
        # event file has already been created. This is important because we use the
        # most recent file in `logdir` as our event file.
        tc.set_logdir(logdir, MODE_TENSORFLOW2)
        return ret

    module.create_summary_file_writer = custom_create_summary_file_writer
    logger.debug(f"Module {module_name} was successfully patched.")


def _patch_event_file_writers(tc):
    module_names = [
        "tensorflow.python.summary.writer.writer",  # TF1
        "tensorboard.summary.writer.event_file_writer",
        "torch.utils.tensorboard.writer",
        "tensorboardX.writer",
    ]

    for module_name in module_names:
        try:
            module = importlib.import_module(module_name)
        except Exception as e:
            logger.debug(f"Module {module_name} not found. Skipping patch...")
            continue

        _patch_event_file_writer(module, tc)
        logger.debug(f"Module {module_name} was successfully patched.")


def _patch_event_file_writer(module, tc):
    _EventFileWriter = module.EventFileWriter

    class CustomEventFileWriter(_EventFileWriter):
        def __init__(self, *args, **kwargs):
            # self.log_dir is set in super().__init__
            super(CustomEventFileWriter, self).__init__(*args, **kwargs)

            logdir = self.get_logdir()
            tc.set_logdir(logdir, MODE_OTHERS)

    module.EventFileWriter = CustomEventFileWriter



import glob
import os
from tempfile import TemporaryDirectory

from transformers import (
    Trainer,
    TrainerCallback,
    TrainerControl,
    TrainerState,
    TrainingArguments,
)

from vessl.internal import api, finish
from vessl.internal import hyperparameters as hp
from vessl.internal import init, log
from vessl.model import create_model, upload_model_volume_file
from vessl.util import logger

# keys which are in summary logs
SUMMARY_KEYS = {
    "train_runtime",
    "train_samples_per_second",
    "train_steps_per_second",
    "train_loss",
    "total_flos",
}


class VesslCallback(TrainerCallback):
    """
    A [`TrainerCallback`] that logs metrics, media, model checkpoints to [VESSL AI](https://vessl.ai/).

    Args:
        access_token(str): Access token to override. Defaults to
                `access_token` from `~/.vessl/config`.
        organization_name(str): Organization name to override. Defaults to
                `organization_name` from `~/.vessl/config`.
        project_name(str): Project name to override. Defaults to
            `project name` from `~/.vessl/config`.
        credentials_file(str): Defaults to None.
        force_update_access_token(bool): True if force update access token,
            False otherwise. Defaults to False.
        upload_model(bool): True if upload model after training finishes. Defaults to False.
        repository_name(str): Repository name to upload model. If it is None but upload_model is True,
            raise an exception. Defaults to None.
    """

    def __init__(
        self,
        access_token: str = None,
        organization_name: str = None,
        project_name: str = None,
        credentials_file: str = None,
        force_update_access_token: bool = False,
        upload_model: bool = False,
        repository_name: str = None,
    ) -> None:
        api.configure(
            access_token=access_token,
            organization_name=organization_name,
            project_name=project_name,
            credentials_file=credentials_file,
            force_update_access_token=force_update_access_token,
        )
        self._initialized = False

        self.upload_model = upload_model
        self.repository_name = repository_name

        if self.upload_model and self.repository_name is None:
            raise ValueError("You must specify the repository name when you want to upload model")

    def setup(self, args: "TrainingArguments", state: "TrainerState", **kwargs):
        """
        Setup the optional VESSL integration.

        If training is performed in VESSL Run, do nothing.
        Otherwise, such as in case of local experiments, initialize the new VESSL Experiment and connect to it.
        """
        self._initialized = True
        if state.is_world_process_zero:
            init()

    def on_train_begin(
        self, args: "TrainingArguments", state: "TrainerState", control: "TrainerControl", **kwargs
    ):
        """Setup VESSL when training begins"""
        if not self._initialized:
            self.setup(args, state)

    def on_log(
        self,
        args: "TrainingArguments",
        state: "TrainerState",
        control: "TrainerControl",
        logs: dict[str, float] = None,
        **kwargs,
    ) -> None:
        if not self._initialized:
            self.setup(args, state)

        # log metrics in the main process only
        if state.is_world_process_zero:
            # if the current step is the max step, and the summary keys are subset of log keys, just skip logging.
            # It is summary log and it will be handled in on_train_end logs
            log_keys = set(logs.keys())
            if state.max_steps != state.global_step or not SUMMARY_KEYS.issubset(log_keys):
                log(payload=logs, step=state.global_step)

    def on_train_end(
        self, args: "TrainingArguments", state: "TrainerState", control: "TrainerControl", **kwargs
    ):
        """Clean up experiment and upload final model when training ends"""
        if self._initialized and state.is_world_process_zero:
            finish()

            # get the last log and if it contains train stats such as train_runtime, train_steps_per_seconds, etc,
            # log to hyperparameter section
            summary_log = state.log_history[-1]
            if api.is_in_run_exec_context():
                for k in summary_log.keys():
                    print(f"{k}: {summary_log[k]}")
            else:
                hp.update(summary_log)

            if self.upload_model:
                with TemporaryDirectory() as temp_dir:
                    temp_trainer = Trainer(
                        args=args, model=kwargs.get("model"), tokenizer=kwargs.get("tokenizer")
                    )
                    temp_trainer.save_model(temp_dir)

                    response = create_model(self.repository_name)
                    if response is None:
                        # if model has not been created successfully, the function returns None
                        logger.exception("There was an error while creating model")
                    else:
                        model_number = response.number
                        for file_path in glob.glob(f"{temp_dir}/*"):
                            upload_model_volume_file(
                                self.repository_name,
                                model_number,
                                file_path,
                                os.path.basename(file_path),
                            )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmCoupon(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'code': 'str',
        'created_by': 'str',
        'created_dt': 'datetime',
        'credit_expiration_seconds': 'int',
        'credit_value': 'int',
        'edges': 'OrmCouponEdges',
        'end_dt': 'datetime',
        'group_name': 'str',
        'id': 'int',
        'immutable_slug': 'str',
        'redeem_current_count': 'int',
        'redeem_max_count': 'int',
        'start_dt': 'datetime',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'code': 'code',
        'created_by': 'created_by',
        'created_dt': 'created_dt',
        'credit_expiration_seconds': 'credit_expiration_seconds',
        'credit_value': 'credit_value',
        'edges': 'edges',
        'end_dt': 'end_dt',
        'group_name': 'group_name',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'redeem_current_count': 'redeem_current_count',
        'redeem_max_count': 'redeem_max_count',
        'start_dt': 'start_dt',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, code=None, created_by=None, created_dt=None, credit_expiration_seconds=None, credit_value=None, edges=None, end_dt=None, group_name=None, id=None, immutable_slug=None, redeem_current_count=None, redeem_max_count=None, start_dt=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """OrmCoupon - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._code = None
        self._created_by = None
        self._created_dt = None
        self._credit_expiration_seconds = None
        self._credit_value = None
        self._edges = None
        self._end_dt = None
        self._group_name = None
        self._id = None
        self._immutable_slug = None
        self._redeem_current_count = None
        self._redeem_max_count = None
        self._start_dt = None
        self._updated_dt = None
        self.discriminator = None

        if code is not None:
            self.code = code
        if created_by is not None:
            self.created_by = created_by
        if created_dt is not None:
            self.created_dt = created_dt
        if credit_expiration_seconds is not None:
            self.credit_expiration_seconds = credit_expiration_seconds
        if credit_value is not None:
            self.credit_value = credit_value
        if edges is not None:
            self.edges = edges
        if end_dt is not None:
            self.end_dt = end_dt
        if group_name is not None:
            self.group_name = group_name
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        if redeem_current_count is not None:
            self.redeem_current_count = redeem_current_count
        if redeem_max_count is not None:
            self.redeem_max_count = redeem_max_count
        if start_dt is not None:
            self.start_dt = start_dt
        if updated_dt is not None:
            self.updated_dt = updated_dt

    @property
    def code(self):
        """Gets the code of this OrmCoupon.  # noqa: E501


        :return: The code of this OrmCoupon.  # noqa: E501
        :rtype: str
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this OrmCoupon.


        :param code: The code of this OrmCoupon.  # noqa: E501
        :type code: str
        """

        self._code = code

    @property
    def created_by(self):
        """Gets the created_by of this OrmCoupon.  # noqa: E501


        :return: The created_by of this OrmCoupon.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this OrmCoupon.


        :param created_by: The created_by of this OrmCoupon.  # noqa: E501
        :type created_by: str
        """

        self._created_by = created_by

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmCoupon.  # noqa: E501


        :return: The created_dt of this OrmCoupon.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmCoupon.


        :param created_dt: The created_dt of this OrmCoupon.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def credit_expiration_seconds(self):
        """Gets the credit_expiration_seconds of this OrmCoupon.  # noqa: E501


        :return: The credit_expiration_seconds of this OrmCoupon.  # noqa: E501
        :rtype: int
        """
        return self._credit_expiration_seconds

    @credit_expiration_seconds.setter
    def credit_expiration_seconds(self, credit_expiration_seconds):
        """Sets the credit_expiration_seconds of this OrmCoupon.


        :param credit_expiration_seconds: The credit_expiration_seconds of this OrmCoupon.  # noqa: E501
        :type credit_expiration_seconds: int
        """

        self._credit_expiration_seconds = credit_expiration_seconds

    @property
    def credit_value(self):
        """Gets the credit_value of this OrmCoupon.  # noqa: E501


        :return: The credit_value of this OrmCoupon.  # noqa: E501
        :rtype: int
        """
        return self._credit_value

    @credit_value.setter
    def credit_value(self, credit_value):
        """Sets the credit_value of this OrmCoupon.


        :param credit_value: The credit_value of this OrmCoupon.  # noqa: E501
        :type credit_value: int
        """

        self._credit_value = credit_value

    @property
    def edges(self):
        """Gets the edges of this OrmCoupon.  # noqa: E501


        :return: The edges of this OrmCoupon.  # noqa: E501
        :rtype: OrmCouponEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmCoupon.


        :param edges: The edges of this OrmCoupon.  # noqa: E501
        :type edges: OrmCouponEdges
        """

        self._edges = edges

    @property
    def end_dt(self):
        """Gets the end_dt of this OrmCoupon.  # noqa: E501


        :return: The end_dt of this OrmCoupon.  # noqa: E501
        :rtype: datetime
        """
        return self._end_dt

    @end_dt.setter
    def end_dt(self, end_dt):
        """Sets the end_dt of this OrmCoupon.


        :param end_dt: The end_dt of this OrmCoupon.  # noqa: E501
        :type end_dt: datetime
        """

        self._end_dt = end_dt

    @property
    def group_name(self):
        """Gets the group_name of this OrmCoupon.  # noqa: E501


        :return: The group_name of this OrmCoupon.  # noqa: E501
        :rtype: str
        """
        return self._group_name

    @group_name.setter
    def group_name(self, group_name):
        """Sets the group_name of this OrmCoupon.


        :param group_name: The group_name of this OrmCoupon.  # noqa: E501
        :type group_name: str
        """

        self._group_name = group_name

    @property
    def id(self):
        """Gets the id of this OrmCoupon.  # noqa: E501


        :return: The id of this OrmCoupon.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmCoupon.


        :param id: The id of this OrmCoupon.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmCoupon.  # noqa: E501


        :return: The immutable_slug of this OrmCoupon.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmCoupon.


        :param immutable_slug: The immutable_slug of this OrmCoupon.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def redeem_current_count(self):
        """Gets the redeem_current_count of this OrmCoupon.  # noqa: E501


        :return: The redeem_current_count of this OrmCoupon.  # noqa: E501
        :rtype: int
        """
        return self._redeem_current_count

    @redeem_current_count.setter
    def redeem_current_count(self, redeem_current_count):
        """Sets the redeem_current_count of this OrmCoupon.


        :param redeem_current_count: The redeem_current_count of this OrmCoupon.  # noqa: E501
        :type redeem_current_count: int
        """

        self._redeem_current_count = redeem_current_count

    @property
    def redeem_max_count(self):
        """Gets the redeem_max_count of this OrmCoupon.  # noqa: E501


        :return: The redeem_max_count of this OrmCoupon.  # noqa: E501
        :rtype: int
        """
        return self._redeem_max_count

    @redeem_max_count.setter
    def redeem_max_count(self, redeem_max_count):
        """Sets the redeem_max_count of this OrmCoupon.


        :param redeem_max_count: The redeem_max_count of this OrmCoupon.  # noqa: E501
        :type redeem_max_count: int
        """

        self._redeem_max_count = redeem_max_count

    @property
    def start_dt(self):
        """Gets the start_dt of this OrmCoupon.  # noqa: E501


        :return: The start_dt of this OrmCoupon.  # noqa: E501
        :rtype: datetime
        """
        return self._start_dt

    @start_dt.setter
    def start_dt(self, start_dt):
        """Sets the start_dt of this OrmCoupon.


        :param start_dt: The start_dt of this OrmCoupon.  # noqa: E501
        :type start_dt: datetime
        """

        self._start_dt = start_dt

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmCoupon.  # noqa: E501


        :return: The updated_dt of this OrmCoupon.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmCoupon.


        :param updated_dt: The updated_dt of this OrmCoupon.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmCoupon):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmCoupon):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseModelServiceGatewayInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'annotations': 'dict[str, str]',
        'enabled': 'bool',
        'endpoint': 'str',
        'ingress_class': 'str',
        'ingress_host': 'str',
        'last_error': 'str',
        'model_service_id': 'int',
        'rules': 'list[OrmModelServiceGatewayTrafficSplitEntry]',
        'status': 'str'
    }

    attribute_map = {
        'annotations': 'annotations',
        'enabled': 'enabled',
        'endpoint': 'endpoint',
        'ingress_class': 'ingress_class',
        'ingress_host': 'ingress_host',
        'last_error': 'last_error',
        'model_service_id': 'model_service_id',
        'rules': 'rules',
        'status': 'status'
    }

    def __init__(self, annotations=None, enabled=None, endpoint=None, ingress_class=None, ingress_host=None, last_error=None, model_service_id=None, rules=None, status=None, local_vars_configuration=None):  # noqa: E501
        """ResponseModelServiceGatewayInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._annotations = None
        self._enabled = None
        self._endpoint = None
        self._ingress_class = None
        self._ingress_host = None
        self._last_error = None
        self._model_service_id = None
        self._rules = None
        self._status = None
        self.discriminator = None

        if annotations is not None:
            self.annotations = annotations
        if enabled is not None:
            self.enabled = enabled
        if endpoint is not None:
            self.endpoint = endpoint
        if ingress_class is not None:
            self.ingress_class = ingress_class
        if ingress_host is not None:
            self.ingress_host = ingress_host
        if last_error is not None:
            self.last_error = last_error
        self.model_service_id = model_service_id
        if rules is not None:
            self.rules = rules
        if status is not None:
            self.status = status

    @property
    def annotations(self):
        """Gets the annotations of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The annotations of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: dict[str, str]
        """
        return self._annotations

    @annotations.setter
    def annotations(self, annotations):
        """Sets the annotations of this ResponseModelServiceGatewayInfo.


        :param annotations: The annotations of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type annotations: dict[str, str]
        """

        self._annotations = annotations

    @property
    def enabled(self):
        """Gets the enabled of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The enabled of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this ResponseModelServiceGatewayInfo.


        :param enabled: The enabled of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type enabled: bool
        """

        self._enabled = enabled

    @property
    def endpoint(self):
        """Gets the endpoint of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The endpoint of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: str
        """
        return self._endpoint

    @endpoint.setter
    def endpoint(self, endpoint):
        """Sets the endpoint of this ResponseModelServiceGatewayInfo.


        :param endpoint: The endpoint of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type endpoint: str
        """

        self._endpoint = endpoint

    @property
    def ingress_class(self):
        """Gets the ingress_class of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The ingress_class of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: str
        """
        return self._ingress_class

    @ingress_class.setter
    def ingress_class(self, ingress_class):
        """Sets the ingress_class of this ResponseModelServiceGatewayInfo.


        :param ingress_class: The ingress_class of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type ingress_class: str
        """

        self._ingress_class = ingress_class

    @property
    def ingress_host(self):
        """Gets the ingress_host of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The ingress_host of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: str
        """
        return self._ingress_host

    @ingress_host.setter
    def ingress_host(self, ingress_host):
        """Sets the ingress_host of this ResponseModelServiceGatewayInfo.


        :param ingress_host: The ingress_host of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type ingress_host: str
        """

        self._ingress_host = ingress_host

    @property
    def last_error(self):
        """Gets the last_error of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The last_error of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: str
        """
        return self._last_error

    @last_error.setter
    def last_error(self, last_error):
        """Sets the last_error of this ResponseModelServiceGatewayInfo.


        :param last_error: The last_error of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type last_error: str
        """

        self._last_error = last_error

    @property
    def model_service_id(self):
        """Gets the model_service_id of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The model_service_id of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: int
        """
        return self._model_service_id

    @model_service_id.setter
    def model_service_id(self, model_service_id):
        """Sets the model_service_id of this ResponseModelServiceGatewayInfo.


        :param model_service_id: The model_service_id of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type model_service_id: int
        """
        if self.local_vars_configuration.client_side_validation and model_service_id is None:  # noqa: E501
            raise ValueError("Invalid value for `model_service_id`, must not be `None`")  # noqa: E501

        self._model_service_id = model_service_id

    @property
    def rules(self):
        """Gets the rules of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The rules of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: list[OrmModelServiceGatewayTrafficSplitEntry]
        """
        return self._rules

    @rules.setter
    def rules(self, rules):
        """Sets the rules of this ResponseModelServiceGatewayInfo.


        :param rules: The rules of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type rules: list[OrmModelServiceGatewayTrafficSplitEntry]
        """

        self._rules = rules

    @property
    def status(self):
        """Gets the status of this ResponseModelServiceGatewayInfo.  # noqa: E501


        :return: The status of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseModelServiceGatewayInfo.


        :param status: The status of this ResponseModelServiceGatewayInfo.  # noqa: E501
        :type status: str
        """

        self._status = status

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseModelServiceGatewayInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseModelServiceGatewayInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmModelServiceRevision(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'arguments': 'dict[str, object]',
        'autoscaler_config': 'dict[str, object]',
        'available_replicas': 'int',
        'created_by_id': 'int',
        'created_dt': 'datetime',
        'edges': 'OrmModelServiceRevisionEdges',
        'id': 'int',
        'immutable_slug': 'str',
        'kernel_image_id': 'int',
        'message': 'str',
        'model_service_id': 'int',
        'number': 'int',
        'ports': 'dict[str, object]',
        'revision_yaml': 'str',
        'spec': 'dict[str, object]',
        'total_replicas': 'int',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'arguments': 'arguments',
        'autoscaler_config': 'autoscaler_config',
        'available_replicas': 'available_replicas',
        'created_by_id': 'created_by_id',
        'created_dt': 'created_dt',
        'edges': 'edges',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'kernel_image_id': 'kernel_image_id',
        'message': 'message',
        'model_service_id': 'model_service_id',
        'number': 'number',
        'ports': 'ports',
        'revision_yaml': 'revision_yaml',
        'spec': 'spec',
        'total_replicas': 'total_replicas',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, arguments=None, autoscaler_config=None, available_replicas=None, created_by_id=None, created_dt=None, edges=None, id=None, immutable_slug=None, kernel_image_id=None, message=None, model_service_id=None, number=None, ports=None, revision_yaml=None, spec=None, total_replicas=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """OrmModelServiceRevision - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._arguments = None
        self._autoscaler_config = None
        self._available_replicas = None
        self._created_by_id = None
        self._created_dt = None
        self._edges = None
        self._id = None
        self._immutable_slug = None
        self._kernel_image_id = None
        self._message = None
        self._model_service_id = None
        self._number = None
        self._ports = None
        self._revision_yaml = None
        self._spec = None
        self._total_replicas = None
        self._updated_dt = None
        self.discriminator = None

        if arguments is not None:
            self.arguments = arguments
        if autoscaler_config is not None:
            self.autoscaler_config = autoscaler_config
        if available_replicas is not None:
            self.available_replicas = available_replicas
        if created_by_id is not None:
            self.created_by_id = created_by_id
        if created_dt is not None:
            self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        if kernel_image_id is not None:
            self.kernel_image_id = kernel_image_id
        if message is not None:
            self.message = message
        if model_service_id is not None:
            self.model_service_id = model_service_id
        if number is not None:
            self.number = number
        if ports is not None:
            self.ports = ports
        self.revision_yaml = revision_yaml
        if spec is not None:
            self.spec = spec
        if total_replicas is not None:
            self.total_replicas = total_replicas
        if updated_dt is not None:
            self.updated_dt = updated_dt

    @property
    def arguments(self):
        """Gets the arguments of this OrmModelServiceRevision.  # noqa: E501


        :return: The arguments of this OrmModelServiceRevision.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._arguments

    @arguments.setter
    def arguments(self, arguments):
        """Sets the arguments of this OrmModelServiceRevision.


        :param arguments: The arguments of this OrmModelServiceRevision.  # noqa: E501
        :type arguments: dict[str, object]
        """

        self._arguments = arguments

    @property
    def autoscaler_config(self):
        """Gets the autoscaler_config of this OrmModelServiceRevision.  # noqa: E501


        :return: The autoscaler_config of this OrmModelServiceRevision.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._autoscaler_config

    @autoscaler_config.setter
    def autoscaler_config(self, autoscaler_config):
        """Sets the autoscaler_config of this OrmModelServiceRevision.


        :param autoscaler_config: The autoscaler_config of this OrmModelServiceRevision.  # noqa: E501
        :type autoscaler_config: dict[str, object]
        """

        self._autoscaler_config = autoscaler_config

    @property
    def available_replicas(self):
        """Gets the available_replicas of this OrmModelServiceRevision.  # noqa: E501


        :return: The available_replicas of this OrmModelServiceRevision.  # noqa: E501
        :rtype: int
        """
        return self._available_replicas

    @available_replicas.setter
    def available_replicas(self, available_replicas):
        """Sets the available_replicas of this OrmModelServiceRevision.


        :param available_replicas: The available_replicas of this OrmModelServiceRevision.  # noqa: E501
        :type available_replicas: int
        """

        self._available_replicas = available_replicas

    @property
    def created_by_id(self):
        """Gets the created_by_id of this OrmModelServiceRevision.  # noqa: E501


        :return: The created_by_id of this OrmModelServiceRevision.  # noqa: E501
        :rtype: int
        """
        return self._created_by_id

    @created_by_id.setter
    def created_by_id(self, created_by_id):
        """Sets the created_by_id of this OrmModelServiceRevision.


        :param created_by_id: The created_by_id of this OrmModelServiceRevision.  # noqa: E501
        :type created_by_id: int
        """

        self._created_by_id = created_by_id

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmModelServiceRevision.  # noqa: E501


        :return: The created_dt of this OrmModelServiceRevision.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmModelServiceRevision.


        :param created_dt: The created_dt of this OrmModelServiceRevision.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this OrmModelServiceRevision.  # noqa: E501


        :return: The edges of this OrmModelServiceRevision.  # noqa: E501
        :rtype: OrmModelServiceRevisionEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmModelServiceRevision.


        :param edges: The edges of this OrmModelServiceRevision.  # noqa: E501
        :type edges: OrmModelServiceRevisionEdges
        """

        self._edges = edges

    @property
    def id(self):
        """Gets the id of this OrmModelServiceRevision.  # noqa: E501


        :return: The id of this OrmModelServiceRevision.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmModelServiceRevision.


        :param id: The id of this OrmModelServiceRevision.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmModelServiceRevision.  # noqa: E501


        :return: The immutable_slug of this OrmModelServiceRevision.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmModelServiceRevision.


        :param immutable_slug: The immutable_slug of this OrmModelServiceRevision.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def kernel_image_id(self):
        """Gets the kernel_image_id of this OrmModelServiceRevision.  # noqa: E501


        :return: The kernel_image_id of this OrmModelServiceRevision.  # noqa: E501
        :rtype: int
        """
        return self._kernel_image_id

    @kernel_image_id.setter
    def kernel_image_id(self, kernel_image_id):
        """Sets the kernel_image_id of this OrmModelServiceRevision.


        :param kernel_image_id: The kernel_image_id of this OrmModelServiceRevision.  # noqa: E501
        :type kernel_image_id: int
        """

        self._kernel_image_id = kernel_image_id

    @property
    def message(self):
        """Gets the message of this OrmModelServiceRevision.  # noqa: E501


        :return: The message of this OrmModelServiceRevision.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this OrmModelServiceRevision.


        :param message: The message of this OrmModelServiceRevision.  # noqa: E501
        :type message: str
        """

        self._message = message

    @property
    def model_service_id(self):
        """Gets the model_service_id of this OrmModelServiceRevision.  # noqa: E501


        :return: The model_service_id of this OrmModelServiceRevision.  # noqa: E501
        :rtype: int
        """
        return self._model_service_id

    @model_service_id.setter
    def model_service_id(self, model_service_id):
        """Sets the model_service_id of this OrmModelServiceRevision.


        :param model_service_id: The model_service_id of this OrmModelServiceRevision.  # noqa: E501
        :type model_service_id: int
        """

        self._model_service_id = model_service_id

    @property
    def number(self):
        """Gets the number of this OrmModelServiceRevision.  # noqa: E501


        :return: The number of this OrmModelServiceRevision.  # noqa: E501
        :rtype: int
        """
        return self._number

    @number.setter
    def number(self, number):
        """Sets the number of this OrmModelServiceRevision.


        :param number: The number of this OrmModelServiceRevision.  # noqa: E501
        :type number: int
        """

        self._number = number

    @property
    def ports(self):
        """Gets the ports of this OrmModelServiceRevision.  # noqa: E501


        :return: The ports of this OrmModelServiceRevision.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this OrmModelServiceRevision.


        :param ports: The ports of this OrmModelServiceRevision.  # noqa: E501
        :type ports: dict[str, object]
        """

        self._ports = ports

    @property
    def revision_yaml(self):
        """Gets the revision_yaml of this OrmModelServiceRevision.  # noqa: E501


        :return: The revision_yaml of this OrmModelServiceRevision.  # noqa: E501
        :rtype: str
        """
        return self._revision_yaml

    @revision_yaml.setter
    def revision_yaml(self, revision_yaml):
        """Sets the revision_yaml of this OrmModelServiceRevision.


        :param revision_yaml: The revision_yaml of this OrmModelServiceRevision.  # noqa: E501
        :type revision_yaml: str
        """

        self._revision_yaml = revision_yaml

    @property
    def spec(self):
        """Gets the spec of this OrmModelServiceRevision.  # noqa: E501


        :return: The spec of this OrmModelServiceRevision.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._spec

    @spec.setter
    def spec(self, spec):
        """Sets the spec of this OrmModelServiceRevision.


        :param spec: The spec of this OrmModelServiceRevision.  # noqa: E501
        :type spec: dict[str, object]
        """

        self._spec = spec

    @property
    def total_replicas(self):
        """Gets the total_replicas of this OrmModelServiceRevision.  # noqa: E501


        :return: The total_replicas of this OrmModelServiceRevision.  # noqa: E501
        :rtype: int
        """
        return self._total_replicas

    @total_replicas.setter
    def total_replicas(self, total_replicas):
        """Sets the total_replicas of this OrmModelServiceRevision.


        :param total_replicas: The total_replicas of this OrmModelServiceRevision.  # noqa: E501
        :type total_replicas: int
        """

        self._total_replicas = total_replicas

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmModelServiceRevision.  # noqa: E501


        :return: The updated_dt of this OrmModelServiceRevision.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmModelServiceRevision.


        :param updated_dt: The updated_dt of this OrmModelServiceRevision.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmModelServiceRevision):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmModelServiceRevision):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmRunExecutionEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'access_token': 'OrmAccessToken',
        'artifacts': 'list[OrmArtifact]',
        'created_by': 'OrmUser',
        'local_experiment': 'OrmExperiment',
        'organization': 'OrmOrganization',
        'pipeline_step_execution': 'OrmPipelineStepExecution',
        'plot_files': 'list[OrmPlotFile]',
        'project': 'OrmProject',
        'run_spec': 'OrmRunSpec',
        'tags': 'list[OrmTag]',
        'volumes': 'list[OrmVolume]',
        'workload': 'OrmWorkload'
    }

    attribute_map = {
        'access_token': 'access_token',
        'artifacts': 'artifacts',
        'created_by': 'created_by',
        'local_experiment': 'local_experiment',
        'organization': 'organization',
        'pipeline_step_execution': 'pipeline_step_execution',
        'plot_files': 'plot_files',
        'project': 'project',
        'run_spec': 'run_spec',
        'tags': 'tags',
        'volumes': 'volumes',
        'workload': 'workload'
    }

    def __init__(self, access_token=None, artifacts=None, created_by=None, local_experiment=None, organization=None, pipeline_step_execution=None, plot_files=None, project=None, run_spec=None, tags=None, volumes=None, workload=None, local_vars_configuration=None):  # noqa: E501
        """OrmRunExecutionEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._access_token = None
        self._artifacts = None
        self._created_by = None
        self._local_experiment = None
        self._organization = None
        self._pipeline_step_execution = None
        self._plot_files = None
        self._project = None
        self._run_spec = None
        self._tags = None
        self._volumes = None
        self._workload = None
        self.discriminator = None

        if access_token is not None:
            self.access_token = access_token
        if artifacts is not None:
            self.artifacts = artifacts
        if created_by is not None:
            self.created_by = created_by
        if local_experiment is not None:
            self.local_experiment = local_experiment
        if organization is not None:
            self.organization = organization
        if pipeline_step_execution is not None:
            self.pipeline_step_execution = pipeline_step_execution
        if plot_files is not None:
            self.plot_files = plot_files
        if project is not None:
            self.project = project
        if run_spec is not None:
            self.run_spec = run_spec
        if tags is not None:
            self.tags = tags
        if volumes is not None:
            self.volumes = volumes
        if workload is not None:
            self.workload = workload

    @property
    def access_token(self):
        """Gets the access_token of this OrmRunExecutionEdges.  # noqa: E501


        :return: The access_token of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmAccessToken
        """
        return self._access_token

    @access_token.setter
    def access_token(self, access_token):
        """Sets the access_token of this OrmRunExecutionEdges.


        :param access_token: The access_token of this OrmRunExecutionEdges.  # noqa: E501
        :type access_token: OrmAccessToken
        """

        self._access_token = access_token

    @property
    def artifacts(self):
        """Gets the artifacts of this OrmRunExecutionEdges.  # noqa: E501


        :return: The artifacts of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: list[OrmArtifact]
        """
        return self._artifacts

    @artifacts.setter
    def artifacts(self, artifacts):
        """Sets the artifacts of this OrmRunExecutionEdges.


        :param artifacts: The artifacts of this OrmRunExecutionEdges.  # noqa: E501
        :type artifacts: list[OrmArtifact]
        """

        self._artifacts = artifacts

    @property
    def created_by(self):
        """Gets the created_by of this OrmRunExecutionEdges.  # noqa: E501


        :return: The created_by of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this OrmRunExecutionEdges.


        :param created_by: The created_by of this OrmRunExecutionEdges.  # noqa: E501
        :type created_by: OrmUser
        """

        self._created_by = created_by

    @property
    def local_experiment(self):
        """Gets the local_experiment of this OrmRunExecutionEdges.  # noqa: E501


        :return: The local_experiment of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmExperiment
        """
        return self._local_experiment

    @local_experiment.setter
    def local_experiment(self, local_experiment):
        """Sets the local_experiment of this OrmRunExecutionEdges.


        :param local_experiment: The local_experiment of this OrmRunExecutionEdges.  # noqa: E501
        :type local_experiment: OrmExperiment
        """

        self._local_experiment = local_experiment

    @property
    def organization(self):
        """Gets the organization of this OrmRunExecutionEdges.  # noqa: E501


        :return: The organization of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmRunExecutionEdges.


        :param organization: The organization of this OrmRunExecutionEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def pipeline_step_execution(self):
        """Gets the pipeline_step_execution of this OrmRunExecutionEdges.  # noqa: E501


        :return: The pipeline_step_execution of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmPipelineStepExecution
        """
        return self._pipeline_step_execution

    @pipeline_step_execution.setter
    def pipeline_step_execution(self, pipeline_step_execution):
        """Sets the pipeline_step_execution of this OrmRunExecutionEdges.


        :param pipeline_step_execution: The pipeline_step_execution of this OrmRunExecutionEdges.  # noqa: E501
        :type pipeline_step_execution: OrmPipelineStepExecution
        """

        self._pipeline_step_execution = pipeline_step_execution

    @property
    def plot_files(self):
        """Gets the plot_files of this OrmRunExecutionEdges.  # noqa: E501


        :return: The plot_files of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: list[OrmPlotFile]
        """
        return self._plot_files

    @plot_files.setter
    def plot_files(self, plot_files):
        """Sets the plot_files of this OrmRunExecutionEdges.


        :param plot_files: The plot_files of this OrmRunExecutionEdges.  # noqa: E501
        :type plot_files: list[OrmPlotFile]
        """

        self._plot_files = plot_files

    @property
    def project(self):
        """Gets the project of this OrmRunExecutionEdges.  # noqa: E501


        :return: The project of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmProject
        """
        return self._project

    @project.setter
    def project(self, project):
        """Sets the project of this OrmRunExecutionEdges.


        :param project: The project of this OrmRunExecutionEdges.  # noqa: E501
        :type project: OrmProject
        """

        self._project = project

    @property
    def run_spec(self):
        """Gets the run_spec of this OrmRunExecutionEdges.  # noqa: E501


        :return: The run_spec of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmRunSpec
        """
        return self._run_spec

    @run_spec.setter
    def run_spec(self, run_spec):
        """Sets the run_spec of this OrmRunExecutionEdges.


        :param run_spec: The run_spec of this OrmRunExecutionEdges.  # noqa: E501
        :type run_spec: OrmRunSpec
        """

        self._run_spec = run_spec

    @property
    def tags(self):
        """Gets the tags of this OrmRunExecutionEdges.  # noqa: E501


        :return: The tags of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: list[OrmTag]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this OrmRunExecutionEdges.


        :param tags: The tags of this OrmRunExecutionEdges.  # noqa: E501
        :type tags: list[OrmTag]
        """

        self._tags = tags

    @property
    def volumes(self):
        """Gets the volumes of this OrmRunExecutionEdges.  # noqa: E501


        :return: The volumes of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: list[OrmVolume]
        """
        return self._volumes

    @volumes.setter
    def volumes(self, volumes):
        """Sets the volumes of this OrmRunExecutionEdges.


        :param volumes: The volumes of this OrmRunExecutionEdges.  # noqa: E501
        :type volumes: list[OrmVolume]
        """

        self._volumes = volumes

    @property
    def workload(self):
        """Gets the workload of this OrmRunExecutionEdges.  # noqa: E501


        :return: The workload of this OrmRunExecutionEdges.  # noqa: E501
        :rtype: OrmWorkload
        """
        return self._workload

    @workload.setter
    def workload(self, workload):
        """Sets the workload of this OrmRunExecutionEdges.


        :param workload: The workload of this OrmRunExecutionEdges.  # noqa: E501
        :type workload: OrmWorkload
        """

        self._workload = workload

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmRunExecutionEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmRunExecutionEdges):
            return True

        return self.to_dict() != other.to_dict()
